/*
 * type_use_case_test.cpp
 *
 *  Created on: May 4, 2012
 *      Author: jb
 */



#define FEAL_OVERRIDE_ASSERTS

#include <flow_gtest.hh>

#include <fstream>
#include <input/input_type.hh>
#include <input/reader_to_storage.hh>
#include <input/accessors.hh>
#include <input/factory.hh>


class Equation {
public:
    static Input::Type::Abstract & get_input_type();

};


class EquationA : public Equation {
public:
	typedef Equation FactoryBaseType;

    static const Input::Type::Record & get_input_rec();
    EquationA(Input::Record rec);

private:
    /// Registrar of class to factory
    static const int registrar;
};


class EquationB : public Equation {
public:
	typedef Equation FactoryBaseType;

	static const Input::Type::Record & get_input_rec();
    EquationB(Input::Record rec);

private:
    /// Registrar of class to factory
    static const int registrar;
};


/**
 * gtest fixture class
 */
class Application : public testing::Test{
public:
    static const Input::Type::Record & get_input_type();

    inline Input::Record input()
    {
    	root_rec = json_reader->get_root_interface<Input::Record>();
    	return root_rec;
    }
    void SetUp() override {
    	std::string f_name = string(UNIT_TESTS_SRC_DIR) + "/input/type_use_case_test.con";
    	FilePath fp(f_name, FilePath::FileType::input_file);
    	json_reader = new Input::ReaderToStorage(fp, get_input_type() );
    }
    void TearDown() override {
    	delete json_reader;
    }
private:
    Input::ReaderToStorage *json_reader;
    Input::Record root_rec;

};

TEST_F(Application, init) {
    using namespace Input;

    FilePath::set_io_dirs(".",".","variant", "./output");

    Array eq_arr = input().val<Array>("equations");

    Iterator<AbstractRecord> it = eq_arr.begin<AbstractRecord>();

    Record x = *it;

    EquationA first_eq( x );

    ++it;

    EquationB second_eq( *it );

}

/******************************************************************************************
 * Implementation - typically in *.cc file.
 *
 */

namespace it = Input::Type;

const it::Record & Application::get_input_type() {
	return it::Record("Application", "Root record of the whole application.")
		// Array of equations with types given by method of class Equation
		.declare_key("equations", it::Array( Equation::get_input_type(), 1, 10 ), it::Default::obligatory(), "")
		.close();
}




const it::Record & EquationA::get_input_rec() {
	return it::Record("EquationA", "For example explicit transport equation solver.")
		.derive_from( const_cast<it::Abstract &>(Equation::get_input_type()) )
		.declare_key("mesh",it::FileName::input(),it::Default::obligatory(),"")
		.declare_key("parameter_a", it::Double(), "")
		.close();
}


const int EquationA::registrar =
		Input::register_class< EquationA, Input::Record >("EquationA") +
		EquationA::get_input_rec().size();



const it::Record & EquationB::get_input_rec() {
	return it::Record("EquationB", "For example implicit transport equation solver.")
		.derive_from( const_cast<it::Abstract &>(Equation::get_input_type()) )
		.declare_key("mesh",it::FileName::input(),it::Default::obligatory(),"")
		.declare_key("parameter_b", it::Integer(), it::Default("111"), "")
		.declare_key("default_str", it::String(), it::Default("\"str value\""), "" )
		.declare_key("substances", it::Array( it::String() ), it::Default::obligatory(), "" )
		.close();
}


const int EquationB::registrar =
		Input::register_class< EquationB, Input::Record >("EquationB") +
		EquationB::get_input_rec().size();



it::Abstract & Equation::get_input_type() {
	return it::Abstract("AbstractEquation","Abstract input Record type for any equation.")
		.close();
}




EquationA::EquationA(Input::Record rec) {
    using namespace Input;

    string mesh_file = rec.val<FilePath>("mesh");
    EXPECT_TRUE(mesh_file.find("some.msh") != string::npos);

    Iterator<double> it = rec.find<double>("parameter_a");
    EXPECT_TRUE(it);
    EXPECT_EQ(3.14, *it);
}






EquationB::EquationB(Input::Record rec) {
    using namespace Input;

    string mesh_file = rec.val<FilePath>("mesh");
    EXPECT_TRUE(mesh_file.find("some.msh") != string::npos);
    int param = rec.val<int>("parameter_b");
    EXPECT_EQ(314, param);

    EXPECT_EQ("str value", rec.val<string>("default_str"));
    EXPECT_TRUE( rec.find<string>("default_str"));

    Array array( rec.val<Array>("substances") );
}



